/*
 * COMU.c
 *
 * Created: 2016/03/29 14:55:40
 *  Author: koichi
 */ 

#include <avr/io.h>
#include "COMMON.h"

void COMU_init(void)
{
	// communication bus with the CPLD (PIO)
	PORTA.OUTSET = 0;
	PORTA.DIRCLR = 0b11111111;
	PORTCFG.MPCMASK = 0b11111111;
	PORTA_PIN0CTRL = PORT_OPC_PULLDOWN_gc;
	
	// control signals for communication with the CPLD (PIO)
	// lower 2 bits: 00:portA, 01:portB, 10:portC
	// read from the PIO when bit 2 is 1.
	// write to the PIO when bit 3 is 1 (and bit 2 is 0).
	// write to the direction register of the PIO when bit 4 is 1 (and bit 2 and bit 3 are 0).
	PORTD.OUTSET = 0;
	PORTD.DIRSET = 0b01111111;

	// control inputs from the CPLD
	// bit 0 is the reset signal from P6
	PORTE.DIRCLR = 0b00001111;
	PORTCFG.MPCMASK = 0b00001111;
	PORTE_PIN0CTRL = PORT_OPC_PULLUP_gc;
}

void COMU_DIR_A(uint8_t d)
{
	VPORT0_OUT = d;
	VPORT0_DIR = 0b11111111;
	VPORT2_OUT = (0 | PIN4_bm);
	VPORT2_OUT &= ~PIN4_bm;
	VPORT0_DIR = 0b00000000;
}

void COMU_DIR_B(uint8_t d)
{
	VPORT0_OUT = d;
	VPORT0_DIR = 0b11111111;
	VPORT2_OUT = (1 | PIN4_bm);
	VPORT2_OUT &= ~PIN4_bm;
	VPORT0_DIR = 0b00000000;
}

void COMU_DIR_C(uint8_t d)
{
	VPORT0_OUT = d;
	VPORT0_DIR = 0b11111111;
	VPORT2_OUT = (2 | PIN4_bm);
	VPORT2_OUT &= ~PIN4_bm;
	VPORT0_DIR = 0b00000000;
}

void COMU_OUT_A(uint8_t d)
{
	VPORT0_OUT = d;
	VPORT0_DIR = 0b11111111;
	VPORT2_OUT = 0 | PIN3_bm;
	VPORT2_OUT &= ~PIN3_bm;
	VPORT0_DIR = 0b00000000;
}

void COMU_OUT_B(uint8_t d)
{
	VPORT0_OUT = d;
	VPORT0_DIR = 0b11111111;
	VPORT2_OUT = 1 | PIN3_bm;
	VPORT2_OUT &= ~PIN3_bm;
	VPORT0_DIR = 0b00000000;
}

void COMU_OUT_C(uint8_t d)
{
	VPORT0_OUT = d;
	VPORT0_DIR = 0b11111111;
	VPORT2_OUT = 2 | PIN3_bm;
	VPORT2_OUT &= ~PIN3_bm;
	VPORT0_DIR = 0b00000000;
}

uint8_t COMU_IN_A(void)
{
	VPORT2_OUT = (0|PIN2_bm);
	nop();
	nop();
	uint8_t d = VPORT0_IN;
	nop();
	VPORT2_OUT &= ~PIN2_bm;
	return d;
}

uint8_t COMU_IN_B(void)
{
	VPORT2_OUT = (1|PIN2_bm);
	nop();
	nop();
	uint8_t d = VPORT0_IN;
	nop();
	VPORT2_OUT &= ~PIN2_bm;
	return d;
}

uint8_t COMU_IN_C(void)
{
	VPORT2_OUT = (2|PIN2_bm);
	nop();
	nop();	
	uint8_t d = VPORT0_IN;
	nop();
	VPORT2_OUT &= ~PIN2_bm;
	return d;
}