/*
 * D88_P31.c
 *
 * Note that only 1D image is supported currently.
 *
 * Created: 2016/05/05 12:08:59
 *  Author: koichi
 */

#include <avr/io.h>

#include "SD.h"
#include "BUFFER.h"
#include "MOUNT.h"
#include "COMMON.h"

// check whether the D88 file is "continuous" or not
// (if it is not continuous, d88 to p31 is really slow...)
uint8_t check_d88_countinuous(struct FILE *fp, uint8_t *err)
{
	FILE_seq_reset(fp);
	FILE_seq_seek_bytes(fp, 0x20, err);
	if (*err) return 0;

	uint32_t adr = 0x000002b0;
	for (uint8_t track = 0; track != 40; track++) {
		uint32_t dwd;
		uint8_t eof;
		FILE_seq_read_dword(fp, &eof, &dwd, err);
		if (*err) return 0;
		if (dwd != adr) return 0;
		FILE_seq_read_dword(fp, &eof, &dwd, err);
		if (*err) return 0;
		adr += 0x1100;
	}
	return 1;
}

void seek_d88_track(struct FILE *fp, uint8_t track, uint8_t *err)
{
	FILE_seq_reset(fp);
	FILE_seq_seek_bytes(fp, (uint32_t)track*4+0x20, err);
	if (*err) return;

	uint8_t eof = 0;
	uint32_t d;

	FILE_seq_read_dword(fp, &eof, &d, err);
	if (*err) return;

	FILE_seq_reset(fp);
	FILE_seq_seek_bytes(fp, d, err);
}

 // convert a d88 file to a p31 file
 void d882p31(struct FILE *p31File, struct FILE *d88File, uint8_t *err)
 {
	if ((!p31File->valid) || (!d88File->valid)) return;

 	uint8_t is_continuous = check_d88_countinuous(d88File, err);
 	if (*err) return;

	for (uint8_t trk=0; trk<80; trk++) {
		if (trk&1) {
			LED2_OFF;
			for (uint8_t sec=0; sec !=16; sec++) {
				SD_changeBuff(buffer2.sd.buf);
				for (uint16_t i=0; i!=512; i++) {
					FILE_seq_write_byte(p31File, 0, err);
					if (*err) {
						SD_changeBuff(buffer1);
						return;
					}
				}
				SD_changeBuff(buffer1);
			}
		} else {
			LED2_ON;
			uint8_t b, eof = 0;
			if (is_continuous && (trk != 0)) {
			} else seek_d88_track(d88File, trk, err);
			if (*err) return;
			for (uint8_t sec=0; sec !=16; sec++) {
				for (uint8_t i=0; i!=16; i++) {
					FILE_seq_read_byte(d88File, &eof, &b, err);
					if (*err) return;
				}
				for (uint16_t i=0; i!=256; i++) {
					FILE_seq_read_byte(d88File, &eof, &b, err);
					SD_changeBuff(buffer2.sd.buf);
					FILE_seq_write_byte(p31File, b, err);
					SD_changeBuff(buffer1);
					if (*err) return;
				}
				SD_changeBuff(buffer2.sd.buf);
				for (uint16_t i=0; i!=256; i++) {
					FILE_seq_write_byte(p31File, 0, err);
					if (*err) {
						SD_changeBuff(buffer1);
						return;
					}
				}
				SD_changeBuff(buffer1);
			}
		}
 	}
	SD_changeBuff(buffer2.sd.buf);
	FILE_seq_write_byte_end(p31File, err);
	SD_changeBuff(buffer1);
}

// convert a p31 file to a d88 file
void p312d88(struct FILE *d88File, struct FILE *p31File, uint8_t *err)
{
	uint8_t b, eof = 0;

	if ((!p31File->valid) || (!d88File->valid)) return;

	FILE_seq_reset(d88File);
	FILE_seq_reset(p31File);

	SD_changeBuff(buffer2.sd.buf);
	for (uint8_t i=0; i!=28; i++) {
		FILE_seq_write_byte(d88File, 0, err);
		if (*err) return;
		
	}
	FILE_seq_write_dword(d88File, 0x0002aab0, err);
	if (*err) return;

	uint32_t adr = 0x000002b0;
	for (uint8_t trk=0; trk<40; trk++) {
		FILE_seq_write_dword(d88File, adr, err);
		FILE_seq_write_dword(d88File, 0, err);
		adr += 0x1100;
	}
	for (uint8_t i=0; i<84; i++) {
		FILE_seq_write_dword(d88File, 0, err);
	}
	SD_changeBuff(buffer1);

	for (uint8_t trk=0; trk<40; trk++) {
		if (trk&1) LED1_OFF;
		else LED1_ON;
		for (uint8_t sec=1; sec !=17; sec++) {
			SD_changeBuff(buffer2.sd.buf);
			FILE_seq_write_byte(d88File, trk, err);
			FILE_seq_write_byte(d88File, 0, err);
			FILE_seq_write_byte(d88File, sec, err);
			FILE_seq_write_byte(d88File, 0x01, err);
			FILE_seq_write_byte(d88File, 0x10, err);
			for (uint8_t i=0; i!=10; i++) {
				FILE_seq_write_byte(d88File, 0, err);
			}
			FILE_seq_write_byte(d88File, 0x01, err);
			SD_changeBuff(buffer1);
			for (uint16_t i=0; i!=256; i++) {
				FILE_seq_read_byte(p31File, &eof, &b, err);
				SD_changeBuff(buffer2.sd.buf);
				FILE_seq_write_byte(d88File, b, err);
				SD_changeBuff(buffer1);
			}
			for (uint16_t i=0; i!=256; i++) {
				FILE_seq_read_byte(p31File, &eof, &b, err);
			}
		}
		for (uint8_t sec=1; sec !=17; sec++) {
			for (uint16_t i=0; i!=512; i++) {
				FILE_seq_read_byte(p31File, &eof, &b, err);
			}
		}
	}
	SD_changeBuff(buffer2.sd.buf);
	FILE_seq_write_byte_end(d88File, err);
	SD_changeBuff(buffer1);
}
